@@ -3,9 +3,9 @@ package Email::Simple::Header;
 use strict;
 use Carp ();
 
-$Email::Simple::Header::VERSION = '1.997_02';
+require Email::Simple;
 
-my $crlf = qr/\x0a\x0d|\x0d\x0a|\x0a|\x0d/;  # We are liberal in what we accept.
+$Email::Simple::Header::VERSION = '2.200_01';
 
 =head1 NAME
 
@@ -30,11 +30,11 @@ minimal interface, and is mostly for private consumption at the moment.
   my $header = Email::Simple::Header->new($head, \%arg);
 
 C<$head> is a string containing a valid email header, or a reference to such a
-string.
+string.  If a reference is passed in, don't expect that it won't be altered.
 
 Valid arguments are:
 
-  crlf - the header's newline; defaults to "\n"
+  crlf - the header's newline; defaults to CRLF
 
 =cut
 
@@ -47,7 +47,7 @@ sub new {
 
   my $head_ref = ref $head ? $head : \$head;
 
-  my $self = { mycrlf => $arg->{crlf} || "\n", };
+  my $self = { mycrlf => $arg->{crlf} || "\x0d\x0a", };
 
   my $headers = $class->_header_to_list($head_ref, $self->{mycrlf});
 
@@ -67,27 +67,32 @@ sub _header_to_list {
 
   my @headers;
 
+  my $crlf = Email::Simple->__crlf_re;
+
   while ($$head =~ m/\G(.+?)$crlf/go) {
     local $_ = $1;
-    if (s/^\s+// or not /^([^:]+):\s*(.*)/) {
+
+    if (/^\s+/ or not /^([^:]+):\s*(.*)/) {
       # This is a continuation line. We fold it onto the end of
       # the previous header.
       next if !@headers;  # Well, that sucks.  We're continuing nothing?
 
-      $headers[-1] .= $headers[-1] ? " $_" : $_;
+      (my $trimmed = $_) =~ s/^\s+//;
+      $headers[-1][0] .= $headers[-1][0] =~ /\S/ ? " $trimmed" : $trimmed;
+      $headers[-1][1] .= "$mycrlf$_";
     } else {
-      push @headers, $1, $2;
+      push @headers, $1, [ $2, $_ ];
     }
   }
 
   return \@headers;
 }
 
-=head2 from_string
-
 =head2 as_string
 
-This returns the stringified header.
+  my $string = $header->as_string(\%arg);
+
+This returns a stringified version of the header.
 
 =cut
 
@@ -98,53 +103,33 @@ This returns the stringified header.
 # kept in blocks prepended to the message.
 
 sub as_string {
-  my ($self) = @_;
+  my ($self, $arg) = @_;
+  $arg ||= {};
 
   my $header_str = '';
 
   my $headers = $self->{headers};
 
+  my $fold_arg = {
+    # at     => (exists $arg->{fold_at} ? $arg->{fold_at} : $self->default_fold_at),
+    # indent => (exists $arg->{fold_indent} ? $arg->{fold_indent} : $self->default_fold_indent),
+    at     => $self->_default_fold_at,
+    indent => $self->_default_fold_indent,
+  };
+
   for (my $i = 0; $i < @$headers; $i += 2) {
-    $header_str .= $self->_header_as_string(@$headers[ $i, $i + 1 ]);
+    if (ref $headers->[ $i + 1 ]) {
+      $header_str .= $headers->[ $i + 1 ][1] . $self->crlf;
+    } else {
+      my $header = "$headers->[$i]: $headers->[$i + 1]";
+
+      $header_str .= $self->_fold($header, $fold_arg);
+    }
   }
 
   return $header_str;
 }
 
-sub _header_as_string {
-  my ($self, $field, $data) = @_;
-
-  # Ignore "empty" headers; this should not be allowed to happen!
-  return '' unless defined $data;
-
-  my $string = "$field: $data";
-
-  return ((length $string > 78) and (lc $field ne 'content-type'))
-    ? $self->_fold($string)
-    : ($string . $self->crlf);
-}
-
-sub _fold {
-  my $self = shift;
-  my $line = shift;
-
-  # We know it will not contain any new lines at present
-  my $folded = "";
-  while ($line) {
-    $line =~ s/^\s+//;
-    if ($line =~ s/^(.{0,77})(\s|\z)//) {
-      $folded .= $1 . $self->crlf;
-      $folded .= " " if $line;
-    } else {
-
-      # Basically nothing we can do. :(
-      $folded .= $line . $self->crlf;
-      last;
-    }
-  }
-  return $folded;
-}
-
 =head2 header_names
 
 This method returns the unique header names found in this header, in no
@@ -170,7 +155,9 @@ they appear in the header.
 sub header_pairs {
   my ($self) = @_;
 
-  return @{ $self->{headers} };
+  my @pairs = map {; _str_value($_) } @{ $self->{headers} };
+
+  return @pairs;
 }
 
 =head2 header
@@ -183,6 +170,8 @@ named field does not appear in the header, this method returns false.
 
 =cut
 
+sub _str_value { return ref $_[0] ? $_[0][0] : $_[0] }
+
 sub header {
   my ($self, $field) = @_;
 
@@ -190,11 +179,11 @@ sub header {
   my $lc_field = lc $field;
 
   if (wantarray) {
-    return map { @$headers[ $_ * 2 + 1 ] }
+    return map { _str_value($headers->[ $_ * 2 + 1 ]) }
       grep { lc $headers->[ $_ * 2 ] eq $lc_field } 0 .. int($#$headers / 2);
   } else {
     for (0 .. int($#$headers / 2)) {
-      return $headers->[ $_ * 2 + 1 ] if lc $headers->[ $_ * 2 ] eq $lc_field;
+      return _str_value($headers->[ $_ * 2 + 1 ]) if lc $headers->[ $_ * 2 ] eq $lc_field;
     }
     return;
   }
@@ -205,10 +194,20 @@ sub header {
   $header->header_set($field => @values);
 
 This method updates the value of the given header.  Existing headers have their
-values set in place.  Additional headers are added at the end.
+values set in place.  Additional headers are added at the end.  If no values
+are given to set, the header will be removed from to the message entirely.
 
 =cut
 
+# Header fields are lines composed of a field name, followed by a colon (":"),
+# followed by a field body, and terminated by CRLF.  A field name MUST be
+# composed of printable US-ASCII characters (i.e., characters that have values
+# between 33 and 126, inclusive), except colon.  A field body may be composed
+# of any US-ASCII characters, except for CR and LF.
+
+# However, a field body may contain CRLF when used in header "folding" and
+# "unfolding" as described in section 2.2.3.
+
 sub header_set {
   my ($self, $field, @data) = @_;
 
@@ -228,7 +227,7 @@ sub header_set {
 
   if (@indices > @data) {
     my $overage = @indices - @data;
-    splice @{$headers}, $_, 2 for reverse @indices[ -$overage, $#indices ];
+    splice @{$headers}, $_, 2 for reverse @indices[ -$overage .. -1 ];
     pop @indices for (1 .. $overage);
   } elsif (@data > @indices) {
     my $underage = @data - @indices;
@@ -253,18 +252,81 @@ This method returns the newline string used in the header.
 
 sub crlf { $_[0]->{mycrlf} }
 
-1;
+# =head2 fold
+# 
+#   my $folded = $header->fold($line, \%arg);
+# 
+# Given a header string, this method returns a folded version, if the string is
+# long enough to warrant folding.  This method is used internally.
+# 
+# Valid arguments are:
+# 
+#   at      - fold lines to be no longer than this length, if possible
+#             if given and false, never fold headers
+#   indent  - indent lines with this string
+# 
+# =cut
 
-__END__
+sub _fold {
+  my ($self, $line, $arg) = @_;
+  $arg ||= {};
+
+  $arg->{at} = $self->_default_fold_at unless exists $arg->{at};
+
+  return $line . $self->crlf unless $arg->{at} and $arg->{at} > 0;
+
+  my $limit  = ($arg->{at} || $self->_default_fold_at) - 1;
+
+  return $line . $self->crlf if length $line <= $limit;
+
+  $arg->{indent} = $self->_default_fold_indent unless exists $arg->{indent};
+
+  my $indent = $arg->{indent} || $self->_default_fold_indent;
+
+  # We know it will not contain any new lines at present
+  my $folded = "";
+  while ($line) {
+    if ($line =~ s/^(.{0,$limit})(\s|\z)//) {
+      $folded .= $1 . $self->crlf;
+      $folded .= $indent if $line;
+    } else {
+      # Basically nothing we can do. :(
+      $folded .= $line . $self->crlf;
+      last;
+    }
+  }
+
+  return $folded;
+}
+
+# =head2 default_fold_at
+# 
+# This method (provided for subclassing) returns the default length at which to
+# try to fold header lines.  The default default is 78.
+# 
+# =cut
+
+sub _default_fold_at { 78 }
+
+# =head2 default_fold_indent
+# 
+# This method (provided for subclassing) returns the default string used to
+# indent folded headers.  The default default is a single space.
+# 
+# =cut
+
+sub _default_fold_indent { " " }
 
 =head1 PERL EMAIL PROJECT
 
 This module is maintained by the Perl Email Project
 
-L<http://emailproject.perl.org/wiki/Email::Simple::Header>
+L<http://emailproject.perl.org/>
 
 =head1 COPYRIGHT AND LICENSE
 
+Copyright 2006-2007 by Ricardo SIGNES
+
 Copyright 2004 by Casey West
 
 Copyright 2003 by Simon Cozens
@@ -273,3 +335,5 @@ This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =cut
+
+1;
